from enum import Enum
from logger import Logger
from typing import Dict, List, Any, Union
import bottle
import json
import os
import time


"""
Формат тест кейса таков:
{
  "<url-путь>": [
    {
      # строка(или null) ответа от сервера
      "response": str,

      # метаданные ответа(могут отсутствовать и тогда будут использованы значения по умолчанию)
      "responsemeta": {
        # строковое перечисление типа возвращаемых данных(по умолчанию dtext)
        "datatype": Enum(str) {dtext, djson, dxml},
         
        # строковое перечисление источника данных для ответа на запрос(по умолчанию stext)
        # если значение sfile, то считаем что данные ответа лежат в файле по пути:
        #    "/linuxcash/cash/servicestub/services/<имя_сервиса>/ + <значение_поля_response>"
        "source": Enum(str) {stext, sfile}
      },

      # возвращаемый статус http(если присутствует, то сервис вернёт его на запрос)
      "status": {
        # код статуса выполнения
        "code": int
      },

      # возвращаемая ошибка http(если присутствует, то сервис вернёт её на запрос)
      "error": {
        # код ошибки
        "code": int
      },

      # заполняемые заголовки в ответе от сервера, задаются в виде словаря. Могут отсутствовать
      "headers": Dict[str, Any]

      # счётчик ответов на запрос. Если отсутствует, то ответ будет возвращаться всегда
      "caseCount": int,

      # таймаут перед ответом сервиса. Может отсутствовать
      "timeout": float
    }
  ]
}

"""


class EDataType(str, Enum):
    dtext = 'text'
    djson = 'json'
    dxml = 'xml'


class ESource(str, Enum):
    stext = 'text'
    sfile = 'file'


class ResponseMeta:
    datatype: Enum
    source: Enum

    def __init__(self, datatype=None, source=None, **kwargs):
        self.datatype = EDataType.dtext if datatype is None else datatype
        self.source = ESource.stext if source is None else source


class Status:
    code: int

    def __init__(self, code=200, **kwargs):
        self.code = code


class HttpError:
    code: int

    def __init__(self, code=200, **kwargs):
        self.code = code


class Headers:
    headers: Dict[str, Any]

    def __init__(self, headers=None, **kwargs):
        self.headers = headers


class Case:
    response: Union[str, None]
    responseMeta: Union[ResponseMeta, None]
    status: Union[Status, None]
    error: Union[HttpError, None]
    headers: Union[Headers, None]
    caseCount: Union[int, None]
    timeout: Union[int, None]

    def __init__(self, response=None, responsemeta=None, status=None, error=None, headers=None, caseCount=None, timeout=None, **kwargs):
        self.response = response
        self.responseMeta = ResponseMeta() if responsemeta is None else ResponseMeta(**responsemeta)
        self.status = None if status is None else Status(**status)
        self.error = None if error is None else HttpError(**error)
        self.headers = None if headers is None else Headers(headers)
        self.count = caseCount
        self.timeout = timeout

    def createResponse(self, serviceDir: str):
        if self.error is not None:
            return bottle.HTTPError(self.error.code)

        # первое - это получение данных
        content = self.response
        if self.responseMeta.source == ESource.sfile:
            with open(os.path.join(serviceDir, self.response)) as f:
                content = f.read()

        # второе - это формирование ответа
        if self.responseMeta.datatype == EDataType.djson:
            content = json.dumps(content)

        # третье - получение ответа и заполнение header'ов
        status = 200 if self.status is None else self.status.code
        return bottle.HTTPResponse(body=content, status=status,
                                   headers={} if self.headers is None else self.headers.headers)


class BasicService(object):
    log: Logger
    servicePath: str  # url-путь сервиса
    serviceDir: str  # путь до директории с сервисом(<имя_сервиса>.py)
    cases: Dict[str, List[Case]]

    def __init__(self, serviceDir):
        self.log = Logger()
        self.servicePath = '/'
        self.cases = {}
        self.serviceDir = serviceDir

    def _loadCases(self, filename: str):
        try:
            with open(os.path.join(self.serviceDir, filename), 'r') as casesFile:
                cases = json.load(casesFile)
            for path in cases:
                self.cases[path] = [Case(**case) for case in cases[path]]
        except Exception as e:
            self.log.error(str(e))

    def _createResponse(self, methodName: str):
        if methodName not in self.cases:
            raise bottle.HTTPError(400, 'Метод "%s" не найден' % methodName)

        if not len(self.cases[methodName]):
            self.log.warn('В кейсах не осталось значений')
            raise bottle.HTTPError(500, 'В кейсах не осталось значений')

        case = self.cases[methodName][0]

        if case.timeout is not None:
            time.sleep(case.timeout)

        response = case.createResponse(self.serviceDir)

        # уменьшить счётчик кейса, если он есть
        if case.count is not None:
            case.count -= 1
            if case.count == 0:
                del self.cases[methodName][0]

        if response is bottle.HTTPError:
            raise response
        return response
