import bottle
import json
import os
import xml.etree.ElementTree as ET

from logger import Logger
from singleton import Singleton
from typing import List


class Egais(metaclass=Singleton):
    exciseBottleAttrs = ['price', 'barcode', 'ean', 'volume']

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()
        servicestub.route('/', method='GET', callback=self.checkServerAvailable)
        servicestub.route('/xml', method='POST', callback=self.getEgaisCheck)

    def checkServerAvailable(self):
        self.log.info('Запрос доступности сервера')
        return self.__getResult('/')

    def getEgaisCheck(self):
        self.log.info('Получаем данные для ЕГАИС')
        xml = bottle.request.forms.get("xml_file")
        try:
            root = ET.fromstring(xml)
            excise = root.findall('Bottle')
            if len(excise) > 0:
                self.log.info('Проверка акцизной продукции')
                Egais.checkExcise(excise)
        except Exception as e:
            self.log.error(e)
            message = 'Некорректные данные чека: cheque.xml'
            self.log.warning(message)
            raise bottle.HTTPError(400, message)

        self.log.info(u'Отправляем ответ от ЕГАИС')
        return self.__getResult('/xml')

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warning('В кейсах не осталось значений')
            raise bottle.HTTPError(self.cases[methodPath].get('code', 400))

        response = None
        if 'response' in self.cases[methodPath][0]:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), self.cases[methodPath][0]['response']), 'rb') as f:
                body = f.read()
            response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status', 200), body=body)
            response.add_header('Content-Type', 'text/xml; charset=utf8')
        elif 'error' in self.cases[methodPath][0]:
            response = bottle.HTTPError(self.cases[methodPath][0]['error'].get('code', 400))
        else:
            raise bottle.HTTPError(400)

        if 'count' in self.cases[methodPath][0]:
            self.cases[methodPath][0]['count'] -= 1
            if self.cases[methodPath][0]['count'] == 0:
                del self.cases[methodPath][0]

        if response is bottle.HTTPError:
            raise response
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))

    @staticmethod
    def checkExcise(xmlNodesList: List[ET.Element]) -> None:
        """ Проверка акцизной продукции на наличие обязательных полей
        Проверка наличия значений и типов не производится,  кроме проверки уникальности поля 'barcode'
        """
        barcodes = {}
        for xmlNode in xmlNodesList:
            for attr in Egais.exciseBottleAttrs:
                if xmlNode.attrib.get(attr) is None:
                    raise bottle.HTTPError(400, 'В акцизной продукции %s' % xmlNode +
                            ' отсутствует необходимый атрибут: %s' % attr)
                if attr == 'barcode':
                    barcodes[xmlNode.attrib.get(attr)] = 1
        if len(xmlNodesList) != len(barcodes):
            raise bottle.HTTPError(400, 'Обнаружено дублирование акцизной продукции')


instance = Egais()
