from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Mobile(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса mobile')
        self.__updateCases()

        servicestub.route('/mobile/client', method='POST', callback=self.client)
        servicestub.route('/mobile/client/balance', method='POST', callback=self.balance)
        servicestub.route('/mobile/client/modify', method='POST', callback=self.modify)
        servicestub.route('/mobile/client/registration', method='POST', callback=self.clientRegistration)

        servicestub.route('/mobile/feed', method='POST', callback=self.feed)
        servicestub.route('/mobile/static/images/<filename:path>', method='GET', name='static', callback=self.image)

        servicestub.route('/mobile/bonushistory', method='POST', callback=self.bonusHistory)
        servicestub.route('/mobile/purchasehistory', method='POST', callback=self.purchaseHistory)
        servicestub.route('/mobile/check', method='POST', callback=self.check)

        servicestub.route('/mobile/verification/request', method='POST', callback=self.verificationRequest)
        servicestub.route('/mobile/verification/verify', method='POST', callback=self.verificationVerify)

        servicestub.route('/mobile/catalog/goods', method='POST', callback=self.goods)
        servicestub.route('/mobile/catalog', method='POST', callback=self.catalog)

    def client(self):
        self.__updateCases()
        payload = bottle.request.json
        phone = payload.get('phone')
        self.log.info('Запрос списка карт по номеру телефона ' + phone)
        methodPath = "/mobile/client/" + phone

        if methodPath not in self.cases:
            methodPath = "/mobile/client/notfound"

        return self.__getResult(methodPath)

    def balance(self):
        self.__updateCases()
        self.log.info('Запрос расширенного баланса по карте')

        return self.__getResult("/mobile/client/balance")
    
    def modify(self):
        self.__updateCases()
        self.log.info('Запрос расширенного баланса по карте')

        return self.__getResult("/mobile/client/modify")

    def clientRegistration(self):
        self.__updateCases()
        params = bottle.request.params
        self.log.info('Регистарция клиента')
        return self.__getResult("/mobile/client/registration")

    def feed(self):
        self.__updateCases()
        params = bottle.request.params

        self.log.info('Получение ленты для номера ' + params.phone)

        return self.__getResult("/mobile/feed")

    def bonusHistory(self):
        self.__updateCases()
        params = bottle.request.params

        self.log.info('Получение истории бонусных операций')

        return self.__getResult("/mobile/bonushistory")

    def purchaseHistory(self):
        self.__updateCases()
        params = bottle.request.params

        self.log.info('Получение истории покупок')

        return self.__getResult("/mobile/purchasehistory")

    def check(self):
        self.__updateCases()
        params = bottle.request.params

        self.log.info('Получение чека')

        return self.__getResult("/mobile/check")

    def image(self, filename: str):
        print("image", filename)
        self.log.info('Изображение %s', filename)

        return bottle.static_file(filename, root=os.path.join(os.path.dirname(os.path.abspath(__file__)), 'images'))

    def verificationRequest(self):
        self.__updateCases()
        params = bottle.request.params

        self.log.info('Запрос кода подтверждения для телефона ' + params.phone)

        return self.__getResult('/mobile/verification/request')

    def verificationVerify(self):
        self.__updateCases()

        payload = bottle.request.json
        phone = payload.get('phone')
        code = payload.get('code')
        self.log.info('Проверка кода ' + code + ' для телефона ' + phone)

        methodPath = '/mobile/verification/verify'
        expectedCode = self.cases[methodPath].get('code', '1234')

        if code != expectedCode:
            return self.__getResult(methodPath + '/incorrectcode')

        return self.__getResult(methodPath)

    def goods(self):
        self.__updateCases()
        payload = bottle.request.json
        id = payload.get('id')
        self.log.info('Получение информации о товаре ' + id)

        methodPath = "/mobile/catalog/goods/" + id

        if methodPath not in self.cases:
            methodPath = "/mobile/catalog/goods/notfound"

        return self.__getResult(methodPath)

    def catalog(self):
        self.__updateCases()
        payload = bottle.request.json
        id = payload.get('id')
        self.log.info('Получение информации о каталоге ' + id)

        methodPath = "/mobile/catalog/" + id

        if methodPath not in self.cases:
            methodPath = "/mobile/catalog/notfound"

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))

        responseIndex = self.cases[methodPath].get('responseIndex', '')

        body = json.dumps(self.cases[methodPath].get('response' + responseIndex))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        try:
            for file in "mobile", "verify", "feed", "purchasehistory", "bonushistory", "check", "registration",\
                        "goods", "catalogs":
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = Mobile()
